@@ -1,7 +1,8 @@
 #!/usr/bin/env python
 # -*- coding: utf-8 -*-
 
-__version__ = "0.4.7"
+__version__ = "0.4.16.11"
 from robotathome.version import *
+from robotathome.helpers import *
 from robotathome.toolbox import *
 from robotathome.log import *
