@@ -4,108 +4,45 @@
 __author__ = "Gregorio Ambrosio"
 __contact__ = "gambrosio[at]uma.es"
 __copyright__ = "Copyright 2021, Gregorio Ambrosio"
-__date__ = "2021/02/22"
+__date__ = "2021/07/27"
 __license__ = "MIT"
 
 import sys
-import time
-import re
 import datetime as dt
 import sqlite3
 import os
 import cv2
 import numpy as np
 import pandas as pd
+import mxnet as mx
 from mxnet import image
 from mxnet import context
+import gluoncv as gcv
 from gluoncv import model_zoo, data, utils
-from matplotlib import pyplot as plt
+# import fiftyone as fo
+# from matplotlib import pyplot as plt
+# import helpers
 import robotathome as rh
 # import fire
 
 
-def time_win2unixepoch(time_stamp):
-    ''' Doctring '''
-    seconds = time_stamp / 10000000
-    epoch = seconds - 11644473600
-    datetime_ = dt.datetime(2000, 1, 1, 0, 0, 0)
-    return datetime_.fromtimestamp(epoch)
-
-
-def time_unixepoch2win(date):
-    ''' Doctring '''
-    match = re.compile(r'^(\d{4})-(\d\d)-(\d\d)T(\d\d):(\d\d):(\d\d)$').match(date)
-    if match:
-        datetime_ = dt.datetime(*map(int, match.groups()))
-        windows_timestamp = (time.mktime(datetime_.timetuple()) + 11644473600) * 10000000
-    else:
-        print("Invalid date format specified: " + date)
-        print("Specify a date and time string in the format: \"yyyy-MM-ddTHH:mm:ss\"")
-        windows_timestamp = 0
-    return windows_timestamp
-
-
-def flat2Dlist(list_):
-    return sum(list_, [])
-
-
-def flatlist(list_):
-    if len(list_) == 0:
-        return list_
-    if isinstance(list_[0], list):
-        return flatlist(list_[0]) + flatlist(list_[1:])
-    return list_[:1] + flatlist(list_[1:])
-
-
-def reverse_dict(dict_):
-    return dict(map(reversed, dict_.items()))
-
-
-def get_log_levels():
-    log_levels_key_no = {}
-    level_values = rh.logger._core.levels.values()
-    for level_value in level_values:
-        log_levels_key_no[level_value.no] = level_value.name
-    log_levels_key_name = reverse_dict(log_levels_key_no)
-    return log_levels_key_no, log_levels_key_name
-
-
-def get_log_level_name(level_no):
-    log_levels_key_no, _ = get_log_levels()
-    return log_levels_key_no[level_no]
-
-
-def get_log_level_no(level_name):
-    _, log_levels_key_name = get_log_levels()
-    return log_levels_key_name[level_name]
-
-
-def current_log_level():
-    """
-    This function returns True if the current logging level is under
-    'level' value
-    """
-    level_no = rh.logger._core.min_level
-    level_name = get_log_level_name(level_no)
-
-    return level_no, level_name
-
-
-def is_being_logged(level_name='DEBUG'):
-    current_log_level_no, _ = current_log_level()
-    return current_log_level_no <= get_log_level_no(level_name)
-
-
-def rename_if_exist(file_name, tail='.bak'):
-    if os.path.isfile(file_name):
-        rh.logger.warning("This file name already exists. Adding .bak")
-        new_file_name = file_name + tail
-        os.rename(file_name, new_file_name)
-
-
 class RobotAtHome():
-    """
-    RobotAtHome class with methods to manage the Robot@Home dataset v2.x.y
+    """RobotAtHome class with methods for Robot@Home dataset v2.x.y
+
+    The RobotAtHome class encapsulates methods to access the RobotAtHome
+    database. <https://doi.org/10.5281/zenodo.4530453>
+
+    Attributes:
+        rh_path (str, optional):
+            root path for robotathome database, usually rh.db
+        wspc_path (str, optional):
+            workspace path where temporary files are stored
+        db_filename (str, optional):
+            default database name
+        rgbd_path (str, optional):
+           path that completes rh_path, where rgbd files are stored
+        scene_path (str, optional):
+           path that coompletes rh_path, where scene files are stored
     """
 
     def __init__(self,
@@ -238,6 +175,41 @@ class RobotAtHome():
         # There is only one view for now
         return self.__rgbd_views
 
+    def query(self, sql, df=True):
+        """Execute a sqlquery over robotathome database
+
+        Parameters
+        ----------
+        sql: can be a string with a sql query or a file name that contains the
+             sql query
+        df:  boolean indicating if result is returned as a DataFrame (True) or
+             as a sqlite row list (False)
+
+        Returns
+        -------
+        ans: a DataFrame or a sqlite row list
+
+        """
+
+
+        if os.path.isfile(sql):
+            script = open(sql, 'r')
+            query = script.read()
+        else:
+            query = sql
+
+        if df:
+            ans = pd.read_sql_query(query, self.__con)
+        else:
+            cur = self.__con.cursor()
+            cur.executescript(query)
+            ans = cur.fetchall()
+
+        if os.path.isfile(sql):
+            script.close()
+
+        return ans
+
     def get_home_session_names(self):
         """
         Return a list with home session names
@@ -280,6 +252,35 @@ class RobotAtHome():
         """
         return self.select_column('name', 'rh_object_types')
 
+    def get_locators(self):
+        """ Return a dataframe table with main indexes values (id and name),
+        i.e., home_session, home, room and home_subsession
+        """
+
+        sql_str = """
+        select
+            home_session_id, rh_home_sessions.name as home_session_name,
+            rh_raw.home_id, rh_homes.name as home_name,
+            rh_raw.room_id, rh_rooms.name as room_name,
+            rh_raw.home_subsession_id
+        from rh_raw
+        inner join rh_home_sessions on home_session_id = rh_home_sessions.id
+        inner join rh_homes on rh_raw.home_id = rh_homes.id
+        inner join rh_rooms on rh_raw.room_id = rh_rooms.id
+        group by
+            home_session_id,
+            rh_raw.home_id,
+            rh_raw.room_id,
+            rh_raw.home_subsession_id
+
+        order by
+            rh_raw.home_session_id
+        """
+
+        df_rows = pd.read_sql_query(sql_str, self.__con)
+
+        return df_rows
+
     def get_sensor_observation_files(self,
                                      source='lblrgbd',
                                      home_session_name='alma-s1',
@@ -319,12 +320,14 @@ class RobotAtHome():
         #     order by t
         #     '''
         # )
-
+        #
         # parms = (home_session_name,
         #          home_subsession,
         #          room_name,
         #          sensor_name)
         # cur.execute(sql_str, parms)
+        # cur.execute(sql_str)
+        # rows = cur.fetchall()
 
         sql_str = (
             f'''
@@ -340,12 +343,156 @@ class RobotAtHome():
         )
         rh.logger.debug(sql_str)
 
-        # cur.execute(sql_str)
-        # rows = cur.fetchall()
         df_rows = pd.read_sql_query(sql_str, self.__con)
 
         return df_rows
 
+    def get_labels_from_lblrgbd(self, so_id):
+        """
+        This function return labels rows for the observations referenced by
+        sensor_observation_id
+
+        SQL query
+
+        select * from rh_lblrgbd_labels
+            where sensor_observation_id = so_id
+
+        Parameters
+        ----------
+        so_id : int
+            The primary key value to identify a row in the table
+            rh_lbl_rgbd_labels.
+
+        Returns
+        -------
+        A dataframe with the query result. An empty dataframe is returned when
+        no rows are available, i.e., when the sensor observation does not
+        belong to rh_lblrgbd (labelled rgbd)
+        """
+
+        # Get a cursor to execute SQLite statements
+        cur = self.__con.cursor()
+
+        # # Build the query
+        # sql_str = (
+        #     '''
+        #     select * from rh_lblrgbd_labels
+        #     where sensor_observation_id = ?
+        #     '''
+        # )
+
+        # parms = (so_id,)
+        # cur.execute(sql_str, parms)
+        # rows = cur.fetchall()
+
+        sql_str = (
+            '''
+            select * from rh_lblrgbd_labels
+            where sensor_observation_id = {}
+            '''.format(so_id)
+        )
+
+        df_rows = pd.read_sql_query(sql_str, self.__con)
+
+        # print(df.shape)
+        # rows = df.to_records()
+        # for row in rows:
+        #     print(row)
+
+        return df_rows
+
+    def __get_mask(self, label_path_file_name):
+        mask = []
+        with open(label_path_file_name, "r") as file_handler:
+            line = file_handler.readline()
+            while line:
+                words = line.strip().split()
+                if words[0][0] != '#':
+                    num_of_labels = int(words[0])
+                    break
+                line = file_handler.readline()
+
+            for i in range(num_of_labels):
+                line = file_handler.readline()
+                words = line.strip().split()
+
+            num_of_rows = 0
+            line = file_handler.readline()
+            while line:
+                num_of_rows += 1
+                words = line.strip().split()
+                mask.append(list(map(int, words)))
+                line = file_handler.readline()
+
+        rh.logger.debug("mask height: {}", len(mask))
+        rh.logger.debug("mask width : {}", len(mask[0]))
+
+        mask = np.array(mask)
+        mask = np.rot90(mask)
+
+        return mask
+
+    def get_mask_from_lblrgbd(self, so_id):
+        """
+        This function 
+
+        Parameters
+        ----------
+
+        Returns
+        -------
+
+        """
+        # Get a cursor to execute SQLite statements
+        # cur = self.__con.cursor()
+
+        # sql_str = (
+        #     '''
+        #     select pth, f3
+        #     from rh_temp_lblrgbd
+        #     where id = {}
+        #     '''.format(so_id)
+        # )
+
+        sql_str = (
+            f'''
+            select pth, f3
+            from rh_temp_lblrgbd
+            where id = {so_id}
+            '''
+        )
+
+        df_rows = pd.read_sql_query(sql_str, self.__con)
+        rh.logger.debug("df_rows.shape: {}", df_rows.shape)
+        # print(df_rows)
+        # print(df_rows.loc[0,"pth"])
+        # print(df_rows.loc[0,"f3"])
+        label_path_file_name = os.path.join(self.__rh_path,
+                                            self.__rgbd_path,
+                                            df_rows.loc[0, "pth"],
+                                            df_rows.loc[0, "f3"])
+
+        rh.logger.debug("label_path_file_name: {}", label_path_file_name)
+        mask = self.__get_mask(label_path_file_name)
+
+        return mask
+
+    def get_label_mask(self, mask, labels):
+        """
+        Returns a binary 2D array (pixels being 1s and 0s)
+        """
+        masks = []
+        for label in labels:
+            arr = mask & (2**(label))
+            np.clip(arr, 0, 1, out=arr)
+            arr = np.uint8(arr[:,2:-2])
+            masks.append(arr)
+        return masks
+
+
+    """
+    OpenCV
+    """
     def get_video_from_rgbd(self,
                             source='lblrgbd',
                             home_session_name='alma-s1',
@@ -545,98 +692,7 @@ class RobotAtHome():
 
         return video_file_name
 
-    def get_labels_from_sensor_observation(self, so_id):
-        """
-        This function return labels rows for the observations referenced by
-        sensor_observation_id
-
-        SQL query
-
-        select * from rh_lblrgbd_labels
-            where sensor_observation_id = so_id
-
-        Parameters
-        ----------
-        so_id : int
-            The primary key value to identify a row in the table
-            rh_lbl_rgbd_labels.
-
-        Returns
-        -------
-        A dataframe with the query result. An empty dataframe is returned when
-        no rows are available, i.e., when the sensor observation does not
-        belong to rh_lblrgbd (lebelled rgbd)
-        """
-
-        # Get a cursor to execute SQLite statements
-        cur = self.__con.cursor()
-
-        # # Build the query
-        # sql_str = (
-        #     '''
-        #     select * from rh_lblrgbd_labels
-        #     where sensor_observation_id = ?
-        #     '''
-        # )
-
-        # parms = (so_id,)
-        # cur.execute(sql_str, parms)
-        # rows = cur.fetchall()
-
-        sql_str = (
-            '''
-            select * from rh_lblrgbd_labels
-            where sensor_observation_id = {}
-            '''.format(so_id)
-        )
-
-        df_rows = pd.read_sql_query(sql_str, self.__con)
-
-        # print(df.shape)
-        # rows = df.to_records()
-        # for row in rows:
-        #     print(row)
-
-        return df_rows
-
-    def __get_mask(self, label_path_file_name):
-        mask = []
-        with open(label_path_file_name, "r") as file_handler:
-            line = file_handler.readline()
-            while line:
-                words = line.strip().split()
-                if words[0][0] != '#':
-                    num_of_labels = int(words[0])
-                    break
-                line = file_handler.readline()
-
-            for i in range(num_of_labels):
-                line = file_handler.readline()
-                words = line.strip().split()
-
-            num_of_rows = 0
-            line = file_handler.readline()
-            while line:
-                num_of_rows += 1
-                words = line.strip().split()
-                mask.append(list(map(int, words)))
-                line = file_handler.readline()
-
-        rh.logger.debug("mask height: {}", len(mask))
-        rh.logger.debug("mask width : {}", len(mask[0]))
-
-        return mask
-
-    def get_image_mask_from_label(self, mask, label_id):
-        arr = np.array(mask)
-        arr = np.rot90(arr)
-        arr = arr & (2**(label_id))
-        np.clip(arr, 0, 1, out=arr)
-        arr = np.uint8(arr)
-        # arr = arr * 255
-        return arr
-
-    def get_mask_from_sensor_observation(self, so_id):
+    def get_rgb_image_from_lblrgbd(self, so_id):
         """
         This function 
 
@@ -645,41 +701,203 @@ class RobotAtHome():
 
         Returns
         -------
+        A BGR cv2 image
 
         """
-        # Get a cursor to execute SQLite statements
-        cur = self.__con.cursor()
 
-        # sql_str = (
-        #     '''
-        #     select pth, f3
-        #     from rh_temp_lblrgbd
-        #     where id = {}
-        #     '''.format(so_id)
-        # )
-
-        sql_str = (
-            f'''
-            select pth, f3
-            from rh_temp_lblrgbd
-            where id = {so_id}
-            '''
-        )
+        sql_str = f"""
+        select pth, f2
+        from rh_temp_lblrgbd
+        where id = {so_id}
+        """
 
         df_rows = pd.read_sql_query(sql_str, self.__con)
-        rh.logger.debug("df_rows.shape: {}", df_rows.shape)        
-        # print(df_rows)
-        # print(df_rows.loc[0,"pth"])
-        # print(df_rows.loc[0,"f3"])
-        label_path_file_name = os.path.join(self.__rh_path,
+        # rh.logger.debug("df_rows.shape: {}", df_rows.shape)
+        rgb_image_path_file_name = os.path.join(self.__rh_path,
                                             self.__rgbd_path,
                                             df_rows.loc[0, "pth"],
-                                            df_rows.loc[0, "f3"])
+                                            df_rows.loc[0, "f2"])
 
-        rh.logger.debug("label_path_file_name: {}", label_path_file_name)
-        mask = self.__get_mask(label_path_file_name)
+        # rh.logger.debug("rgb_image_path_file_name: {}",
+        #                 rgb_image_path_file_name)
+        bgr_img = cv2.imread(rgb_image_path_file_name, cv2.IMREAD_COLOR)
 
-        return mask
+        return bgr_img
+
+    def get_depth_image_from_lblrgbd(self, so_id):
+        """
+        This function
+
+        Parameters
+        ----------
+
+        Returns
+        -------
+        A gray levels cv2 image
+
+        """
+
+        sql_str = f"""
+        select pth, f1
+        from rh_temp_lblrgbd
+        where id = {so_id}
+        """
+
+        df_rows = pd.read_sql_query(sql_str, self.__con)
+        rh.logger.debug("df_rows.shape: {}", df_rows.shape)
+        rgb_image_path_file_name = os.path.join(self.__rh_path,
+                                            self.__rgbd_path,
+                                            df_rows.loc[0, "pth"],
+                                            df_rows.loc[0, "f1"])
+
+        rh.logger.debug("rgb_image_path_file_name: {}",
+                        rgb_image_path_file_name)
+        img = cv2.imread(rgb_image_path_file_name, cv2.IMREAD_COLOR)
+
+        return cv2.cvtColor(img, cv2.COLOR_BGR2GRAY)
+
+    def lblrgbd_plot_labels(self, so_id):
+        img = self.get_rgb_image_from_lblrgbd(so_id)
+        labels = self.get_labels_from_lblrgbd(so_id)
+        rh.logger.debug("labels: {}", labels)
+        mask = self.get_mask_from_lblrgbd(so_id)
+        label_mask = self.get_label_mask(mask, labels['local_id'])
+        alpha = 0.7
+        out_img, colors = rh.overlay_mask(cv2.cvtColor(img, cv2.COLOR_BGR2RGB),
+                                          label_mask,
+                                          alpha)
+        rh.plot_mask(out_img, labels['name'], colors)
+
+
+    """
+    MXNet + GluoncV
+    """
+
+    def lblrgbd_rgb_image_object_detection(self, so_id,
+                                           model='yolo3_darknet53_coco'):
+        bgr_img = self.get_rgb_image_from_lblrgbd(so_id)
+        chw_img, class_names, nn_out = rh.object_detection_with_gluoncv(
+            cv2.cvtColor(bgr_img, cv2.COLOR_BGR2RGB),
+            model
+        )
+        return bgr_img, chw_img, class_names, nn_out
+
+    def lblrgbd_object_detection(self,
+                                 source='lblrgbd',
+                                 home_session_name='alma-s1',
+                                 home_subsession=0,
+                                 room_name='alma_masterroom1',
+                                 sensor_name='RGBD_1',
+                                 video_file_name=None,
+                                 model='yolo3_darknet53_coco',
+                                 gpu=False
+                                 ):
+        """
+        This functions ...
+        """
+        rows = self.get_sensor_observation_files(source,
+                                                 home_session_name,
+                                                 home_subsession,
+                                                 room_name,
+                                                 sensor_name)
+
+        # Computing frames per second
+        num_of_frames = len(rows)
+        seconds = (rows.iloc[-1]['t'] - rows.iloc[0]['t']) / 10**7
+        frames_per_second = num_of_frames / seconds
+        rh.logger.debug("frames per second: {:.2f}", frames_per_second)
+
+        # Get frame size
+        img = self.get_rgb_image_from_lblrgbd(rows.iloc[0]['id'])
+        img_h, img_w, _ = img.shape
+
+        # Opening video file
+        if video_file_name is None:
+            video_file_name = ''.join(
+                [
+                    home_session_name,
+                    '_', str(home_subsession),
+                    '_', room_name,
+                    '_', sensor_name,
+                    '_by_', model,
+                    dt.datetime.now().strftime("_%Y%m%d%H%M%S"),
+                    '.avi'
+                ]
+            )
+        fourcc = cv2.VideoWriter_fourcc(*'MJPG')
+        video_path_file_name = os.path.abspath(os.path.join(self.__wspc_path,
+                                                            video_file_name)
+                                               )
+        rh.rename_if_exist(video_path_file_name)
+
+        out = cv2.VideoWriter(video_path_file_name,
+                              fourcc,
+                              frames_per_second,
+                              (img_w, img_h))
+
+        ##############################################
+        #                     NN
+        ##############################################
+
+        yolo_models = rh.get_yolo_models()
+        rcnn_models = rh.get_rcnn_models()
+
+        if model not in yolo_models + rcnn_models:
+            raise Exception(f"Sorry, the model '{model}' is not allowed")
+
+        #  Set context to cpu or gpu
+        ctx_ = mx.context.gpu() if gpu else mx.context.cpu()
+
+        # Load Pretrained Model from the CV model zoo
+        net = gcv.model_zoo.get_model(model,
+                                      pretrained=True,
+                                      ctx=ctx_)
+        class_names_ = net.classes
+        nn_out_list = []
+        i = 0
+        for _, row in rows.iterrows():
+            img = self.get_rgb_image_from_lblrgbd(row['id'])
+
+            i += 1
+            if rh.is_being_logged('INFO'):
+                sys.stdout.write("\rProcessing frame %i of %i" % (i, len(rows)))
+                sys.stdout.flush()
+
+            short_edge_size = min(img.shape[0:2])
+            if model in yolo_models:
+                trnf_img, _ = gcv.data.transforms.presets.yolo.transform_test(mx.nd.array(img),
+                                                                              short=short_edge_size)
+            if model in rcnn_models:
+                trnf_img, _ = gcv.data.transforms.presets.rcnn.transform_test(mx.nd.array(img),
+                                                                              short=short_edge_size)
+            class_ids, scores, bounding_boxs = net(trnf_img)
+
+            df_nn_out = rh.nn_out2df(class_ids, scores, bounding_boxs)
+            nn_out_list.append(df_nn_out)
+
+            utils.viz.cv_plot_bbox(img,
+                                   bounding_boxs[0],
+                                   scores[0],
+                                   class_ids[0],
+                                   class_names=class_names_,
+                                   thresh=0.2,
+                                   linewidth=1)
+            if rh.is_being_logged():
+                cv2.imshow('Debug mode (press q to exit)', img)
+                if cv2.waitKey(1) & 0xFF == ord('q'):
+                    break
+
+            out.write(img)
+
+        out.release()
+
+        if rh.is_being_logged():
+            cv2.destroyAllWindows()
+
+        df_nn_out = pd.DataFrame(nn_out_list, columns=['class_ids',
+                                                       'scores',
+                                                       'bounding_boxs'])
+        return df_nn_out, video_file_name
 
     def process_with_yolo(self,
                           source='lblrgbd',
@@ -695,6 +913,9 @@ class RobotAtHome():
         This functions ...
         """
 
+        rh.logger.warning("This function is DEPRECATED, use lblrgbd_object_detection()")
+        rh.logger.warning("with model='yolo3_darknet53_coco' instead ")
+
         rows = self.get_sensor_observation_files(source,
                                                  home_session_name,
                                                  home_subsession,
@@ -833,6 +1054,9 @@ class RobotAtHome():
         This functions ...
         """
 
+        rh.logger.warning("This function is DEPRECATED, use lblrgbd_object_detection()")
+        rh.logger.warning("with model='faster_rcnn_resnet50_v1b_coco' instead ")
+
         rows = self.get_sensor_observation_files(source,
                                                  home_session_name,
                                                  home_subsession,
@@ -958,6 +1182,66 @@ class RobotAtHome():
         return df_nn_out, video_file_name
 
 
+    """
+    Fiftyone
+    """
+
+    """
+    Lab
+    """
+
+    def create_table_linking_observations_and_lblrgbd(self):
+        """
+        This function 
+
+        Parameters
+        ----------
+
+        Returns
+        -------
+
+        """
+        # Get a cursor to execute SQLite statements
+        # cur = self.__con.cursor()
+
+        sql_str_lblrgbd = (
+            f'''
+            select id, sensor_id
+            from rh_lblrgbd
+            order by id
+            limit 860
+            '''
+        )
+
+        sql_str_observations = (
+            f'''
+            select id, sensor_id
+            from rh_observations
+            order by id
+            -- limit 20
+            '''
+        )
+
+        df_rows_lblrgbd = pd.read_sql_query(sql_str_lblrgbd, self.__con)
+        df_rows_observations = pd.read_sql_query(sql_str_observations, self.__con)
+
+        # for row_lblrgbd in df_rows_lblrgbd.itertuples(index=False):
+        #     print(row_lblrgbd)
+        #     print(row_observation)
+
+        i_obs = 0
+        for i_lbl in range(df_rows_lblrgbd.shape[0]):
+            if df_rows_lblrgbd['sensor_id'][i_lbl] == df_rows_observations['sensor_id'][i_obs]:
+                print(df_rows_lblrgbd['id'][i_lbl], df_rows_observations['id'][i_obs])
+                i_obs += 1
+
+    def get_objects_from_sensor_observations(self, so_id):
+        """TODO """
+        pass
+
+
+
+
 def main():
 
     """ Docstring """
